==> reactx-script-1.1.2.tgz: package/scripts/prepareProxy.js <==
/*
 * @Description: proxy代理配置
 * @Author: lzc
 * @Date: 2020/5/6 10:07:49
 */
const address = require('address');
const fs = require('fs');
const path = require('path');
const url = require('url');
const chalk = require('chalk');

function resolveLoopback(proxy) {
  const o = url.parse(proxy);
  o.host = undefined;
  if (o.hostname !== 'localhost') {
    return proxy;
  }

  try {
    if (!address.ip()) {
      o.hostname = '127.0.0.1';
    }
  } catch (_ignored) {
    o.hostname = '127.0.0.1';
  }
  return url.format(o);
}

function onProxyError(proxy) {
  return (err, req, res) => {
    const host = req.headers && req.headers.host;
    console.log(
      chalk.red('Proxy error:') +
      ' Could not proxy request ' +
      chalk.cyan(req.url) +
      ' from ' +
      chalk.cyan(host) +
      ' to ' +
      chalk.cyan(proxy) +
      '.'
    );
    console.log(
      'See https://nodejs.org/api/errors.html#errors_common_system_errors for more information (' +
      chalk.cyan(err.code) +
      ').'
    );
    console.log();

    // And immediately send the proper error response to the client.
    // Otherwise, the request will eventually timeout with ERR_EMPTY_RESPONSE on the client side.
    if (res.writeHead && !res.headersSent) {
      res.writeHead(500);
    }
    res.end(
      'Proxy error: Could not proxy request ' +
      req.url +
      ' from ' +
      host +
      ' to ' +
      proxy +
      ' (' +
      err.code +
      ').'
    );
  };
}


function prepareProxy(proxy, appPublicFolder, servedPathname) {
  if (!proxy) {
    return undefined;
  }

  if (Object.prototype.toString.call(proxy) !== '[object Object]') {
    return undefined;
  }

  if (Object.keys(proxy).length === 0) {
    return undefined;
  }

  const sockPath = process.env.WDS_SOCKET_PATH || '/sockjs-node';
  const isDefaultSockHost = !process.env.WDS_SOCKET_HOST;

  return Object.entries(proxy).map(([match, opts]) => {
    let target;
    if (process.platform === 'win32') {
      target = resolveLoopback(opts.target);
    } else {
      target = opts.target;
    }

    function mayProxy(pathname) {
      const maybePublicPath = path.resolve(
        appPublicFolder,
        pathname.replace(new RegExp('^' + servedPathname), '')
      );
      const isPublicFileRequest = fs.existsSync(maybePublicPath);
      const isWdsEndpointRequest =
        isDefaultSockHost && pathname.startsWith(sockPath);
      return !(isPublicFileRequest || isWdsEndpointRequest);
    }


    return {
      target,
      pathRewrite: opts.pathRewrite,
      logLevel: 'warn',
      context: (pathname, req) => {
        return (
          req.method !== 'GET' ||
          (mayProxy(pathname) &&
            req.headers.accept &&
            req.headers.accept.indexOf('text/html') === -1)
        );
      },
      onProxyRes: function onProxyReq(proxyRes, req) {
        proxyRes.headers['x-proxy-by'] = 'reactx-proxy';
        proxyRes.headers['x-proxy-match'] = match;
        proxyRes.headers['x-proxy-target'] = target;

        let distTarget = target;
        if (target && target.endsWith('/')) {
          distTarget = target.replace(/\/$/, '');
        }
        proxyRes.headers['x-proxy-target-path'] = distTarget + req.url;
      },
      onError: function onError(err, req, res) {
        // proxy server error can't trigger onProxyRes
        res.writeHead(500, {
          'x-proxy-by': 'ice-proxy',
          'x-proxy-match': match,
          'x-proxy-target': target,
        });
        res.end(`proxy server error: ${err.message}`);
      },
      secure: false,
      changeOrigin: true,
      ws: true,
      xfwd: true
    }
  });
}

module.exports = prepareProxy;

==> reactx-script-1.1.3.tgz: package/scripts/prepareProxy.js <==
/*
 * @Description: proxy代理配置
 * @Author: lzc
 * @Date: 2020/5/6 10:07:49
 */
const address = require('address');
const fs = require('fs');
const path = require('path');
const url = require('url');
const chalk = require('chalk');

function resolveLoopback(proxy) {
  const o = url.parse(proxy);
  o.host = undefined;
  if (o.hostname !== 'localhost') {
    return proxy;
  }

  try {
    if (!address.ip()) {
      o.hostname = '127.0.0.1';
    }
  } catch (_ignored) {
    o.hostname = '127.0.0.1';
  }
  return url.format(o);
}

function onProxyError(proxy) {
  return (err, req, res) => {
    const host = req.headers && req.headers.host;
    console.log(
      chalk.red('Proxy error:') +
      ' Could not proxy request ' +
      chalk.cyan(req.url) +
      ' from ' +
      chalk.cyan(host) +
      ' to ' +
      chalk.cyan(proxy) +
      '.'
    );
    console.log(
      'See https://nodejs.org/api/errors.html#errors_common_system_errors for more information (' +
      chalk.cyan(err.code) +
      ').'
    );
    console.log();

    // And immediately send the proper error response to the client.
    // Otherwise, the request will eventually timeout with ERR_EMPTY_RESPONSE on the client side.
    if (res.writeHead && !res.headersSent) {
      res.writeHead(500);
    }
    res.end(
      'Proxy error: Could not proxy request ' +
      req.url +
      ' from ' +
      host +
      ' to ' +
      proxy +
      ' (' +
      err.code +
      ').'
    );
  };
}


function prepareProxy(proxy, appPublicFolder, servedPathname) {
  if (!proxy) {
    return undefined;
  }

  if (Object.prototype.toString.call(proxy) !== '[object Object]') {
    return undefined;
  }

  if (Object.keys(proxy).length === 0) {
    return undefined;
  }

  const sockPath = process.env.WDS_SOCKET_PATH || '/sockjs-node';
  const isDefaultSockHost = !process.env.WDS_SOCKET_HOST;

  const result = {};

  Object.entries(proxy).forEach(([match, opts]) => {
    let target;
    if (process.platform === 'win32') {
      target = resolveLoopback(opts.target);
    } else {
      target = opts.target;
    }

    function mayProxy(pathname) {
      const maybePublicPath = path.resolve(
        appPublicFolder,
        pathname.replace(new RegExp('^' + servedPathname), '')
      );
      const isPublicFileRequest = fs.existsSync(maybePublicPath);
      const isWdsEndpointRequest =
        isDefaultSockHost && pathname.startsWith(sockPath);
      return !(isPublicFileRequest || isWdsEndpointRequest);
    }

    result[match] =  {
      target,
      pathRewrite: opts.pathRewrite,
      logLevel: 'warn',
      context: (pathname, req) => {
        return (
          req.method !== 'GET' ||
          (mayProxy(pathname) &&
            req.headers.accept &&
            req.headers.accept.indexOf('text/html') === -1)
        );
      },
      onProxyRes: function onProxyReq(proxyRes, req) {
        proxyRes.headers['x-proxy-by'] = 'reactx-proxy';
        proxyRes.headers['x-proxy-match'] = match;
        proxyRes.headers['x-proxy-target'] = target;

        let distTarget = target;
        if (target && target.endsWith('/')) {
          distTarget = target.replace(/\/$/, '');
        }
        proxyRes.headers['x-proxy-target-path'] = distTarget + req.url;
      },
      onError: function onError(err, req, res) {
        // proxy server error can't trigger onProxyRes
        res.writeHead(500, {
          'x-proxy-by': 'ice-proxy',
          'x-proxy-match': match,
          'x-proxy-target': target,
        });
        res.end(`proxy server error: ${err.message}`);
      },
      secure: false,
      changeOrigin: true,
      ws: true,
      xfwd: true
    }
    // return
  });

  return result;
}

module.exports = prepareProxy;
